"""!define - Returns the oxford dictionary definition of the given term"""
import os
import re
import requests

def define(word):
    app_id = os.environ.get('OXFORD_APP_ID', '')
    app_key = os.environ.get('OXFORD_APP_KEY', '')

    if not app_id or not app_key:
        return "Please set the OXFORD_APP_ID and OXFORD_APP_KEY environment variables " \
               "to valid (free) Oxford dictionary API keys: " \
               "https://developer.oxforddictionaries.com/"

    # LANGUAGE CODES AVAILABLE HERE: https://developer.oxforddictionaries.com/documentation/languages
    language = os.environ.get('OXFORD_LANG_CODE', 'en')

    info = {
        'language': language.lower(),
        'word': word.lower()
    }
    headers = {
        'app_id': app_id,
        'app_key': app_key
    }

    if len(word.split(" ")) > 1:
        return "Please only attempt to define a single word at a time!"

    request_url = 'https://od-api.oxforddictionaries.com:443/api/v1/entries/{language}/{word}'.format(**info)
    definition_url = 'https://{language}.oxforddictionaries.com/definition/{word}'.format(**info)

    result = requests.get(request_url, headers=headers)
    if result.status_code == 404:
        return "Oxford has no definition for {0}. If you're searching for the plural, try the singular term".format(
            word)
    elif result.status_code != 200:
        return "Something went wrong when searching for _{0}_! Please try again later".format(word)
    else:
        data = result.json()
        try:
            info['example'] = data['results'][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['examples'][0][
                'text']
        except KeyError:
            info['example'] = 'Well this is awkward, no example is given for this definition... :scream:'
        info['definition'] = data['results'][0]['lexicalEntries'][0]['entries'][0]['senses'][0]['definitions'][0]
        info['definition_url'] = definition_url

        return """*Oxford Dictionary Definition for _{word}_*:
>{definition}

*Example usage*:
_{example}_

{definition_url}""".format(**info)

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!define (.*)", text)
    if not match:
        return

    word = match[0]
    return define(word)



import os
import re

DIR = os.path.dirname(os.path.realpath(__file__))
VCF = """BEGIN:VCARD
VERSION:3.0
N:Abel;Alayna;;;
FN:Alayna Abel
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2407157056
EMAIL;type=INTERNET;type=WORK:alayna@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Auclair;Mike;;;
FN:Mike Auclair
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4014516037
EMAIL;type=INTERNET;type=WORK:mike.auclair@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Balboni;Jeff;;;
FN:Jeff Balboni
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4065997488
EMAIL;type=INTERNET;type=WORK:jeff@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Baliff;Rachel;;;
FN:Rachel Baliff
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:240-463-8495
EMAIL;type=INTERNET;type=WORK:rachel@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Bonenberger;Brian;;;
FN:Brian Bonenberger
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:810-599-0918
EMAIL;type=INTERNET;type=WORK:brian@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Carelock;Nikki;;;
FN:Nikki Carelock
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:8323985056
EMAIL;type=INTERNET;type=WORK:nikki@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Chapman;Danny;;;
FN:Danny Chapman
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:401-451-3151
EMAIL;type=INTERNET;type=WORK:danny@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Cizmadia;Dee;;;
FN:Dee Cizmadia
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:310.270.3930
EMAIL;type=INTERNET;type=WORK:dee@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Cloud;Daniel;;;
FN:Daniel Cloud
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2028027073
EMAIL;type=INTERNET;type=WORK:daniel.cloud@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Clyde;Nick;;;
FN:Nick Clyde
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:530-559-2575
EMAIL;type=INTERNET;type=WORK:nick@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Dawson;Alastair;;;
FN:Alastair Dawson
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:714-595-3281
EMAIL;type=INTERNET;type=WORK:alastair@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Dengo;Sophia;;;
FN:Sophia Dengo
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:8325663655
EMAIL;type=INTERNET;type=WORK:sophia@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:DiMartino;Robert;;;
FN:Robert DiMartino
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:513-827-0573
EMAIL;type=INTERNET;type=WORK:robert.dimartino@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Doan;Eugene;;;
FN:Eugene Doan
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:857-526-6580
EMAIL;type=INTERNET;type=WORK:eugene@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Ellena;Laura;;;
FN:Laura Ellena
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:858-699-0544
EMAIL;type=INTERNET;type=WORK:laura@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Fairhead;Bob;;;
FN:Bob Fairhead
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:6177840700
EMAIL;type=INTERNET;type=WORK:bob@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Fettet;Louis;;;
FN:Louis Fettet
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:5042020991
EMAIL;type=INTERNET;type=WORK:louis@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Follows;Karey;;;
FN:Karey Follows
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7035091617
EMAIL;type=INTERNET;type=WORK:karey@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Fromberg;Oren;;;
FN:Oren Fromberg
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2404868091
EMAIL;type=INTERNET;type=WORK:oren@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Gansen;Chris;;;
FN:Chris Gansen
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7345784242
EMAIL;type=INTERNET;type=WORK:chris@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Gardner;Joe;;;
FN:Joe Gardner
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:512-791-8200
EMAIL;type=INTERNET;type=WORK:joe.gardner@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Gershman;Greg;;;
FN:Greg Gershman
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4439282961
EMAIL;type=INTERNET;type=WORK:greg@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Greim;Cat;;;
FN:Cat Greim
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4127217317
EMAIL;type=INTERNET;type=WORK:cat@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Gryth;Brian;;;
FN:Brian Gryth
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:3037485447
EMAIL;type=INTERNET;type=WORK:brian.gryth@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Guse;Nathan;;;
FN:Nathan Guse
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:9203901269
EMAIL;type=INTERNET;type=WORK:nathan@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Gwinn;Katie;;;
FN:Katie Gwinn
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:5639209683
EMAIL;type=INTERNET;type=WORK:katie@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Hawkins;Spencer;;;
FN:Spencer Hawkins
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:5713090621
EMAIL;type=INTERNET;type=WORK:spencer@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Hein;Shawna;;;
FN:Shawna Hein
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:5302774925
EMAIL;type=INTERNET;type=WORK:shawna@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Holland;Aubrey;;;
FN:Aubrey Holland
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:617-290-8159
EMAIL;type=INTERNET;type=WORK:aubrey@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Hsu;Claire;;;
FN:Claire Hsu
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7188012354
EMAIL;type=INTERNET;type=WORK:claire@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:James;Alexis;;;
FN:Alexis James
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2156204651
EMAIL;type=INTERNET;type=WORK:alexis@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Johnson;Carl;;;
FN:Carl Johnson
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:8085990232
EMAIL;type=INTERNET;type=WORK:carl@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Johnson;Chris;;;
FN:Chris Johnson
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:8054785327
EMAIL;type=INTERNET;type=WORK:chris.johnson@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Julian;Jonathan;;;
FN:Jonathan Julian
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4438031427
EMAIL;type=INTERNET;type=WORK:jonathan@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Karshenas;Kam;;;
FN:Kam Karshenas
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2176372227
EMAIL;type=INTERNET;type=WORK:kam@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Kassemi;James;;;
FN:James Kassemi
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:5054140621
EMAIL;type=INTERNET;type=WORK:james.kassemi@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:King;Brian;;;
FN:Brian King
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:12024257630
EMAIL;type=INTERNET;type=WORK:brian.king@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Koski;Ken;;;
FN:Ken Koski
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:651-497-1094
EMAIL;type=INTERNET;type=WORK:ken@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Kutil;Ben;;;
FN:Ben Kutil
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4104409565
EMAIL;type=INTERNET;type=WORK:ben@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Lewis;Elizabeth;;;
FN:Elizabeth Lewis
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7039661887
EMAIL;type=INTERNET;type=WORK:elizabeth.lewis@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Li;Lihan Li;;;
FN:Lihan Li Li
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7036188660
EMAIL;type=INTERNET;type=WORK:lihan@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Lindo;Mo;;;
FN:Mo Lindo
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:3019191674
EMAIL;type=INTERNET;type=WORK:mo@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Linn;Stacy;;;
FN:Stacy Linn
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:5419685218
EMAIL;type=INTERNET;type=WORK:stacy@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Manning;Christopher;;;
FN:Christopher Manning
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:773-290-8012
EMAIL;type=INTERNET;type=WORK:christopher.manning@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Martinez;Austin;;;
FN:Austin Martinez
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2177141336
EMAIL;type=INTERNET;type=WORK:austin@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:McMonagle;Joseph;;;
FN:Joseph McMonagle
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:484 686 4499
EMAIL;type=INTERNET;type=WORK:joe@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Meek;Wryen;;;
FN:Wryen Meek
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:6787015005
EMAIL;type=INTERNET;type=WORK:wryen@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Mejeur;Curtis;;;
FN:Curtis Mejeur
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:6162936808
EMAIL;type=INTERNET;type=WORK:curtis@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Mill;Bill;;;
FN:Bill Mill
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:860-882-3587
EMAIL;type=INTERNET;type=WORK:bill@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Miller;Michael;;;
FN:Michael Miller
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:3013333333
EMAIL;type=INTERNET;type=WORK:michael@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Miller Sharkey;Slayanna;;;
FN:Slayanna Miller Sharkey
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:8285515435
EMAIL;type=INTERNET;type=WORK:leanna@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Million;Savannah;;;
FN:Savannah Million
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2067075447
EMAIL;type=INTERNET;type=WORK:@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Murray;Scott;;;
FN:Scott Murray
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:8643146872
EMAIL;type=INTERNET;type=WORK:scott@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Nagle;Ryan;;;
FN:Ryan Nagle
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7739887258
EMAIL;type=INTERNET;type=WORK:ryan@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Neelbauer;Juliana;;;
FN:Juliana Neelbauer
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2023303907
EMAIL;type=INTERNET;type=WORK:juliana@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Oberhaus;Kristin;;;
FN:Kristin Oberhaus
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:712-470-0087
EMAIL;type=INTERNET;type=WORK:kristin.oberhaus@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Olson;Mark;;;
FN:Mark Olson
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4435621041
EMAIL;type=INTERNET;type=WORK:mark@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:O'Neil;Dan;;;
FN:Dan O'Neil
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7739606045
EMAIL;type=INTERNET;type=WORK:danx@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Phan;Cindy;;;
FN:Cindy Phan
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:9169958395
EMAIL;type=INTERNET;type=WORK:cindy@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Rhein;James;;;
FN:James Rhein
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7739515944
EMAIL;type=INTERNET;type=WORK:james@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Roueche;Rachael;;;
FN:Rachael Roueche
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:703 489 0705
EMAIL;type=INTERNET;type=WORK:rachael.roueche@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Ryan;Bill;;;
FN:Bill Ryan
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:9787299758
EMAIL;type=INTERNET;type=WORK:bill.ryan@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Sahni;Mickin;;;
FN:Mickin Sahni
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:7037171679
EMAIL;type=INTERNET;type=WORK:mickin@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Shyong;Ben;;;
FN:Ben Shyong
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4085059891
EMAIL;type=INTERNET;type=WORK:ben.shyong@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Smith;Graham;;;
FN:Graham Smith
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:267-386-5979
EMAIL;type=INTERNET;type=WORK:graham@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Smith;Paul;;;
FN:Paul Smith
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:773-934-4607
EMAIL;type=INTERNET;type=WORK:paul@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Smith;Winnie;;;
FN:Winnie Smith
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:3028980775
EMAIL;type=INTERNET;type=WORK:winnie@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Steinmetz;Amrom;;;
FN:Amrom Steinmetz
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:9176275884
EMAIL;type=INTERNET;type=WORK:amrom@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Sutton;Michael;;;
FN:Michael Sutton
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:3013790889
EMAIL;type=INTERNET;type=WORK:michael.sutton@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Swain;Veronica;;;
FN:Veronica Swain
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:609-802-4586
EMAIL;type=INTERNET;type=WORK:veronica@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Szekeresh;Sarah-Jaine;;;
FN:Sarah-Jaine Szekeresh
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:(513) 652-6524
EMAIL;type=INTERNET;type=WORK:sarah-jaine@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Szeluga;Chris;;;
FN:Chris Szeluga
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:9089388795
EMAIL;type=INTERNET;type=WORK:chris.szeluga@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Taylor;Alex;;;
FN:Alex Taylor
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4012266317
EMAIL;type=INTERNET;type=WORK:alex@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Topper;Ian;;;
FN:Ian Topper
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:9376812200
EMAIL;type=INTERNET;type=WORK:ian@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Valarida;Christopher;;;
FN:Christopher Valarida
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:503 302 0306
EMAIL;type=INTERNET;type=WORK:chris.valarida@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Veal;Rachel;;;
FN:Rachel Veal
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:571-244-9368
EMAIL;type=INTERNET;type=WORK:rachel.veal@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Vinograd;Patrick;;;
FN:Patrick Vinograd
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:650-804-2530
EMAIL;type=INTERNET;type=WORK:patrick@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Walker;Kristin;;;
FN:Kristin Walker
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:540-797-8853
EMAIL;type=INTERNET;type=WORK:kristin@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Weber;Caitlin;;;
FN:Caitlin Weber
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:503-367-9483
EMAIL;type=INTERNET;type=WORK:caitlin@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Wilkerson;Rob;;;
FN:Rob Wilkerson
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:443 745 1069
EMAIL;type=INTERNET;type=WORK:rob@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Williams;Carolyn;;;
FN:Carolyn Williams
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:2404238529
EMAIL;type=INTERNET;type=WORK:carolyn@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Wohl;Jeremy;;;
FN:Jeremy Wohl
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:415 994 5074
EMAIL;type=INTERNET;type=WORK:jeremy@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Wolfert;Patrick;;;
FN:Patrick Wolfert
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:5413602784
EMAIL;type=INTERNET;type=WORK:patrick.wolfert@adhocteam.us
END:VCARD
BEGIN:VCARD
VERSION:3.0
N:Woodard;Mel;;;
FN:Mel Woodard
ORG:Ad Hoc LLC
TEL;type=CELL;type=VOICE:4252811675
EMAIL;type=INTERNET;type=WORK:mel@adhocteam.us
END:VCARD"""

def phone(name):
    cards = ["BEGIN:{}".format(x) for x in re.split("^BEGIN:", VCF, flags=re.M) if x]
    matches = [c for c in cards if all(n in c for n in name.split())]

    if not matches:
        return "not able to find that person, sorry"

    result = []
    for card in matches:
        fn, phone = re.search("FN:(.*?)\n.*TEL.*?([\d\-]*)\n", card, flags=re.S).groups()
        result.append("*{}*: {}".format(fn, phone))

    return "\n".join(result)


def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!phone (.*)", text)
    if not match:
        return

    searchterm = match[0]
    return phone(searchterm)

on_bot_message = on_message



class LimboServer(object):
    def __init__(self, slack, config, hooks, db):
        self.slack = slack
        self.config = config
        self.hooks = hooks
        self.db = db

    def query(self, sql, *params):
        c = self.db.cursor()
        c.execute(sql, params)
        rows = c.fetchall()
        c.close()
        self.db.commit()
        return rows



"""!wiki <topic> returns a wiki link for <topic>"""
import re
try:
    from urllib import quote
except ImportError:
    from urllib.request import quote

import requests
from bs4 import BeautifulSoup

def wiki(searchterm):
    """return the top wiki search result for the term"""
    searchterm = quote(searchterm)

    url = "https://en.wikipedia.org/w/api.php?action=query&list=search&srsearch={0}&format=json"
    url = url.format(searchterm)

    result = requests.get(url).json()

    pages = result["query"]["search"]

    # try to reject disambiguation pages
    pages = [p for p in pages if 'may refer to' not in p["snippet"]]

    if not pages:
        return ""

    page = quote(pages[0]["title"].encode("utf8"))
    link = "http://en.wikipedia.org/wiki/{0}".format(page)

    r = requests.get("http://en.wikipedia.org/w/api.php?format=json&action=parse&page={0}".format(page)).json()
    soup = BeautifulSoup(r["parse"]["text"]["*"], "html5lib")
    p = soup.find('p').get_text()
    p = p[:8000]

    return u"{0}\n{1}".format(p, link)

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!wiki (.*)", text)
    if not match:
        return

    searchterm = match[0]
    return wiki(searchterm.encode("utf8"))

on_bot_message = on_message



"""!hub [-r <repository>] <command> <arguments> is a command line interface to github

For all commands, hub will assume that you are using the channel's default
repository unless you specify it explicitly with -r.

To set a channel's default repository, use `!hub setdefault <repo>`

To view a channel's default repository, use `!hub getdefault`

Commands:

* `issues`: display the 5 most recent open issues in the repository

    ex: `!hub issues`
* `issue`: display a particular issue

    ex: `!hub issue 6`
* `create`: create an issue with the title given by <arguments>

    ex: `!hub create Title of a bug I found`
* `search`: search the issues for a repository

    ex: `!hub search bot` will return the first 5 issues containing "bot" in
        the default repository
* `pulls`: display the 5 most recent open pull requests

    ex: `!hub pulls`
* `pull`: display a particular pull request

    ex: `!hub pull 55`
    """

import argparse
import json
import os
import re

import requests

HUB_URL = 'https://api.github.com/{0}'

class Github(object):
    def __init__(self, username, password):
        self.auth = username, password
        if not any(self.auth): self.auth = None

    def _get(self, url_fragment, **params):
        return requests.get(
                 HUB_URL.format(url_fragment),
                 auth=self.auth,
                 params=params
               )

    def _post(self, url_fragment, data={}, **params):
        return requests.post(
                 HUB_URL.format(url_fragment),
                 auth=self.auth,
                 data=data,
                 params=params
               )

    def issues(self, repo):
        # defaults to only open issues
        res = self._get('repos/{0}/issues'.format(repo))
        if res.status_code == 200:
            return res.json()

    def issue(self, repo, n):
        res = self._get('repos/{0}/issues/{1}'.format(repo, n))
        if res.status_code == 200:
            return res.json()

    def create_issue(self, repo, title, body=''):
        res = self._post(
                'repos/{0}/issues'.format(repo),
                data=json.dumps({
                    "title": title,
                    "body": body}))
        if res.status_code == 201:
            return res.json()

    def search_issue_in_repo(self, repo, query):
        return self._get(
                'search/issues',
                q="{0} repo:{1}".format(query, repo)).json()

    def pull_requests(self, repo):
        res = self._get('repos/{0}/pulls'.format(repo))
        if res.status_code == 200:
            return res.json()

    def pull(self, repo, n):
        res = self._get('repos/{0}/pulls/{1}'.format(repo, n))
        if res.status_code == 200:
            return res.json()

    def get_all_repos(self):
        repos = self._get('user/repos', per_page=100)
        repo_names = [repo["full_name"] for repo in repos.json()]
        last = re.findall(r'rel="last"', repos.headers['link'])

        page = 2
        while last:
            repos = self._get('user/repos', per_page=100, page=page)
            repo_names += [repo["full_name"] for repo in repos.json()]
            last = re.findall(r'rel="last"', repos.headers['link'])
            page += 1

        return repo_names

# create an authed github object
HUB = Github(os.environ.get("GITHUB_USER"), os.environ.get("GITHUB_PASS"))

# Gather all repo names available to the authed user. Eventually this will
# need to be refreshed; but for now just assume this is good enough.
# ALL_REPOS = HUB.get_all_repos()

def format_issue(issue_json, verbose=False):
    d = {
        "author_icon": issue_json["user"]["avatar_url"],
        "author_name": issue_json["user"]["login"],
        "author_link": issue_json["user"]["html_url"],
        "fallback": issue_json["title"],
        "title": "[{0}] {1}".format(issue_json["number"], issue_json["title"]),
        "title_link": issue_json["html_url"],
        "color": "good"
    }

    if verbose:
        d["text"] = issue_json["body"]

    return d

def format_pull(pull_json):
    pull_json["s"] = "s" if pull_json["commits"] > 1 else ""
    text = "{body}\n>  _*{commits}* commit{s} *{additions}* ++ " \
           "*{deletions}* -- *{changed_files}* changed_".format(
                **pull_json)
    d = {
        "author_icon": pull_json["user"]["avatar_url"],
        "author_name": pull_json["user"]["login"],
        "author_link": pull_json["user"]["html_url"],
        "fallback": pull_json["title"],
        "title": "[{0}] {1}".format(pull_json["number"], pull_json["title"]),
        "title_link": pull_json["html_url"],
        "color": "good",
        "mrkdwn_in": ["text"],
        "text": text,
    }

    return d

def get_default_repo(server, room):
    rows = server.query('''
        SELECT repo FROM github_room_repo_defaults WHERE room=?''', room)

    if rows:
        return rows[0][0]
    return None

def set_default_repo(server, room, repo):
    server.query('''
        INSERT INTO github_room_repo_defaults(room, repo)
        VALUES (?, ?)''', room, repo)

def issues(repo, _):
    issues = HUB.issues(repo)
    if issues is None:
        return "Unable to find repository {0}".format(repo)

    l = len(issues)
    if l == 0:
        return "0 open issues on repository {0}".format(repo)
    elif l > 5:
        text = "{0} open issues, showing the 5 most recent".format(l)
    else:
        text = "{0} open issues".format(l)

    attachments = json.dumps([format_issue(i) for i in issues[:5]])

    return {
        "attachments": attachments,
        "text": text,
    }

def pulls(repo, _):
    pulls = HUB.pull_requests(repo)
    if pulls is None:
        return "Unable to find repository {0}".format(repo)

    l = len(pulls)
    if l == 0:
        return "0 open pull requests on repository {0}".format(repo)
    elif l > 5:
        text = "{0} open pull requests, showing the 5 most recent".format(l)
    else:
        text = "{0} open pull requests".format(l)

    attachments = json.dumps([format_issue(p) for p in pulls[:5]])

    return {
        "attachments": attachments,
        "text": text,
    }

def issue(repo, body):
    n = body[0]
    issue = HUB.issue(repo, n)
    if not issue:
        return "Unable to find issue #{0} in repo {1}".format(n, repo)

    return {
        "attachments": json.dumps([format_issue(issue, verbose=True)]),
        "text": "",
    }

def pull_request(repo, body):
    n = body[0]
    pull = HUB.pull(repo, n)
    if not issue:
        return "Unable to find pull request #{0} in repo {1}".format(n, repo)

    return {
        "attachments": json.dumps([format_pull(pull)]),
        "text": "",
    }

def create_issue(repo, body):
    title = ' '.join(body)
    issue = HUB.create_issue(repo, title)
    if not issue:
        return "Unable to create issue in repo {0}".format(repo)

    attachment = json.dumps([format_issue(issue)])

    return {
        "attachments": attachment,
        "text": "",
    }

def search(repo, body):
    query = ' '.join(body)
    response = HUB.search_issue_in_repo(repo, query)

    if response["total_count"] == 0:
        return {
            "text": "sorry, no issues found"
        }

    issues = response["items"]
    attachments = json.dumps([format_issue(i) for i in issues[:5]])
    text = "Found {0} items".format(response["total_count"])
    return {
        "attachments": attachments,
        "text": text
    }

def getdefault(repo, _):
    return "Default repo for this room is `{0}`. " \
           "To change it, run `!hub setdefault <repo_name>`".format(repo)

COMMANDS = {
    "issues": issues,
    "pulls": pulls,
    "issue": issue,
    "pull": pull_request,
    "create": create_issue,
    "search": search,
    "getdefault": getdefault
}

def github(server, room, cmd, body, repo):
    # If repo wasn't passed in explicitly, grab it from the database
    if not repo:
        repo = get_default_repo(server, room)

    # If we still couldn't find one in the database, either it's a command to
    # set it or we can instrcut the user how to do so
    if not repo:
        if cmd == "setdefault":
            set_default_repo(server, room, body[0])
            return "Default repo for this room set to `{0}`".format(body[0])
        else:
            return "Unable to find default repo for this channel. "\
                   "Run `!hub setdefault <repo_name>`"

    try:
        command_func = COMMANDS[cmd]
        return command_func(repo, body)
    except KeyError:
        return

# Only run create_database on this module's first execution
FIRST=True
def create_database(server):
    server.query('''
        CREATE TABLE IF NOT EXISTS github_room_repo_defaults
            (room text, repo text)''')
    FIRST=False

ARGPARSE = argparse.ArgumentParser()
ARGPARSE.add_argument('-r', dest="repo")
ARGPARSE.add_argument('command', nargs=1)
ARGPARSE.add_argument('body', nargs='*')

def on_message(msg, server):
    if FIRST:
        create_database(server)

    text = msg.get("text", "")
    match = re.findall(r"!hub\s*(.*)", text)
    if not match:
        return

    # If given -h or -v, argparse will try to quit. Don't let it.
    try:
        ns = ARGPARSE.parse_args(match[0].split(' '))
    except SystemExit:
        return __doc__
    command = ns.command[0]

    # if the user calls !hub with no arguments, print help
    if not len(command):
        return __doc__

    kwargs = github(server, msg["channel"], command, ns.body, ns.repo)

    # if github() didn't return anything, or returned any non-dict arg,
    # just return it
    if not kwargs or not isinstance(kwargs, dict):
        return kwargs

    # otherwise, post the message via the slack API; this lets us use fancy
    # formatting rather than the plain formatting the RTM API allows
    server.slack.post_message(
            msg['channel'],
            '',
            as_user=server.slack.username,
            **kwargs)

on_bot_message = on_message



"""$<ticker symbol> for a quote on a stock price"""
from __future__ import print_function
import logging
import re
try:
    from urllib import quote
except ImportError:
    from urllib.request import quote

from bs4 import BeautifulSoup
import requests

logger = logging.getLogger(__name__)

def stockprice(ticker):
    url = "https://www.google.com/finance?q={0}"
    soup = BeautifulSoup(requests.get(url.format(quote(ticker))).text, "html5lib")

    try:
        company, ticker = re.findall(u"^(.+?)\xa0\xa0(.+?)\xa0", soup.text, re.M)[0]
    except IndexError:
        logging.info("Unable to find stock {0}".format(ticker))
        return ""
    price = soup.select("#price-panel .pr span")[0].text
    change, pct = soup.select("#price-panel .nwp span")[0].text.split()
    time = " ".join(soup.select(".mdata-dis")[0].parent.text.split()[:4])
    pct.strip('()')

    emoji = ":chart_with_upwards_trend:" if change.startswith("+") \
            else ":chart_with_downwards_trend:"

    return "{0} {1} {2}: {3} {4} {5} {6} {7}".format(
            emoji, company, ticker, price, change, pct, time, emoji)


def on_message(msg, server):
    text = msg.get("text", "")
    basic_re = r"\B\$([a-zA-Z]\w{0,3}(?:\.\w{1,3})?)\b"
    # slack linkifies some tickers, which results in: $<http://wbc.ax|wbc.ax>
    link_re = r"\$<http://([a-zA-Z]\w{0,3}(?:\.\w{1,3})?)\|"
    matches = re.findall(r"{}|{}".format(basic_re, link_re, text), text)
    if not matches:
        return

    tickers = (''.join(list(x)).encode("utf8") for x in matches)
    prices = (stockprice(ticker) for ticker in tickers)
    return "\n".join(p for p in prices if p)

on_bot_message = on_message



"""!search <query> will return the top google result for that query (!google is an alias)"""
from bs4 import BeautifulSoup
import re
try:
    from urllib import quote, unquote
except ImportError:
    from urllib.request import quote, unquote
import requests


def google(q):
    query = quote(q)
    url = "https://encrypted.google.com/search?q={0}".format(query)
    soup = BeautifulSoup(requests.get(url).text, "html5lib")

    answer = soup.findAll("h3", attrs={"class": "r"})
    if not answer:
        return ":crying_cat_face: Sorry, google doesn't have an answer for you :crying_cat_face:"

    try:
        return unquote(re.findall(r"q=(.*?)&", str(answer[0]))[0])
    except IndexError:
        # in this case there is a first answer without a link, which is a
        # google response! Let's grab it and display it to the user.
        return ' '.join(answer[0].stripped_strings)

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!(?:google|search) (.*)", text)
    if not match:
        return

    return google(match[0])

on_bot_message = on_message



def on_member_joined_channel(msg, server):
    if msg.get("channel") == "C0LKKMZC0":
        print("posting message to user {}".format(msg['user']))
        server.slack.post_message(msg['user'],
            "Welcome to Ad Hoc's #public channel! We're here for your technical questions about completing the homework. If you have non-technical questions, contact us at recruiting.group@adhocteam.us. Please use your real identity in all communications.",
            username="Ad Hoc Welcome Bot",
            icon_emoji=":adhoc:")



"""Log all messages to the database

Only active if the LIMBO_LOG_EVERYTHING environment variable is set"""
import os

DO_LOG = os.environ.get("LIMBO_LOG_EVERYTHING", False)

def on_message(msg, server):
    if DO_LOG:
        server.query("INSERT INTO log VALUES (?, ?, ?, ?, ?)",
                     msg["text"], msg["user"], msg["ts"], msg["team"], msg["channel"])

def on_init(server):
    if DO_LOG:
        server.query("""
CREATE TABLE IF NOT EXISTS log
    (msg STRING, sender STRING, time STRING, team STRING, channel STRING)
""")

on_bot_message = on_message



# -*- coding: utf-8 -*-
"""!banner <text> [--font <font>] [-l] return a banner displaying <text>. Use the -l option to list available fonts"""

import argparse
import re
import pyfiglet

FIGLET = pyfiglet.Figlet()
FONTS = FIGLET.getFonts()

ARGPARSE = argparse.ArgumentParser()
ARGPARSE.add_argument('--font')
ARGPARSE.add_argument('-l', action='store_const', const=True)
ARGPARSE.add_argument('bannertext', nargs='*')

def make_banner(query):
    # Slack turns -- into an emdash; un-turn it
    query = query.replace(u"\u2014", u"--")

    ns = ARGPARSE.parse_args(query.split(" "))
    if ns.l:
        return "```{0}```".format(", ".join(FONTS))
    font = ns.font or "standard"

    if font not in FONTS:
        return "Unable to find font {0}".format(font)

    banner = pyfiglet.figlet_format(" ".join(ns.bannertext), font=font).rstrip()
    if not banner:
        return

    return "```{0}```".format(banner)

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!banner (.*)", text)
    if not match:
        return

    return make_banner(match[0])

on_bot_message = on_message



# -*- coding: utf-8 -*-
"""
!glossary `term` returns the definition for `term` or "I don't know that term."
!glossary add `term`: `definition` adds the definition to limbo's glossary
!glossary remove `term` removes `term` from the glossary
!glossary search `term` to search all entries for `term`
"""

import re

def on_init(server):
    server.query("""
CREATE TABLE IF NOT EXISTS glossary(term text, definition text)
""")

def get(term, server):
    results = server.query("""
SELECT term, definition FROM glossary WHERE term LIKE ?
""", term)
    return results[0] if results else None

def search(term, server):
    wildterm = "%{}%".format(term)
    results = server.query("""
select term, definition from glossary WHERE term LIKE ? OR definition LIKE ?;
""", wildterm, wildterm)
    if not results:
        return "No results found for {}".format(term)
    else:
        return "\n".join("*{}*: {}".format(*result) for result in results)

def add(term, definition, server):
    results = get(term, server)
    if not results:
        server.query("""
INSERT INTO glossary(term, definition) VALUES (?, ?)
""", term, definition)
        return "Successfully added {}".format(term)
    else:
        server.query("""
UPDATE glossary SET definition=? WHERE term=?
        """, definition, results[0])
        return "Successfully updated {}".format(term)

def remove(term, server):
    results = get(term, server)
    if not results:
        return "No definition found for {}".format(term)
    else:
        # if case is different, stick with the version in the DB.
        # this prevents separate definitions for FOO and Foo
        term = results[0]

    server.query("""
DELETE FROM glossary WHERE term=?
""", term)

    return "Removed definition for {}".format(term)

def lookup(term, server):
    results = get(term, server)
    if not results:
        return ("No definition found for {}. Add a definition with "
        "'!glossary add <term>: <definition>'.".format(term))

    return "*{}*: {}".format(*results)

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.search(r'!gloss(ary)? (add|remove\s+|search)?([^:]*)(.*)?', text, re.IGNORECASE)

    if not match:
        return

    groups = match.groups()
    if groups[1]:
        action = groups[1].strip()
        if action == 'add':
            term = groups[2].strip()
            definition = groups[3].lstrip(':').strip()
            return add(term, definition, server)
        elif action == 'remove':
            return remove(groups[2], server)
        elif action == 'search':
            return search(groups[2], server)
    else:
        return lookup(groups[2], server)

on_bot_message = on_message



"""!gif <search term> return a random result from the  google gif search result for <search term>"""

try:
    from urllib import quote
except ImportError:
    from urllib.request import quote
import re
import requests
from random import shuffle

def unescape(url):
    # for unclear reasons, google replaces url escapes with \x escapes
    return url.replace(r"\x", "%")

def gif(searchterm, unsafe=False):
    searchterm = quote(searchterm)

    safe = "&safe=" if unsafe else "&safe=active"
    searchurl = "https://www.google.com/search?tbs=itp:animated&tbm=isch&q={0}{1}".format(searchterm, safe)

    # this is an old iphone user agent. Seems to make google return good results.
    useragent = "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Versio  n/4.0.5 Mobile/8A293 Safari/6531.22.7"

    result = requests.get(searchurl, headers={"User-agent": useragent}).text

    gifs = list(map(unescape, re.findall(r"var u='(.*?)'", result)))
    shuffle(gifs)

    if gifs:
        return gifs[0]
    else:
        return ""

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!gif (.*)", text)
    if not match:
        return

    searchterm = match[0]
    return gif(searchterm.encode("utf8"))

on_bot_message = on_message



#!/usr/bin/env python
from __future__ import print_function
import copy
import functools
from glob import glob
import importlib
import logging
import os
import re
import sqlite3
import sys
import time
import traceback

from .slack import SlackClient, SlackConnectionError, SlackLoginError
from .server import LimboServer
from .fakeserver import FakeServer

CURDIR = os.path.abspath(os.path.dirname(__file__))
DIR = functools.partial(os.path.join, CURDIR)

PYTHON3 = sys.version_info[0] > 2

logger = logging.getLogger(__name__)

class InvalidPluginDir(Exception):
    def __init__(self, plugindir):
        message = "Unable to find plugin dir {0}".format(plugindir)
        super(InvalidPluginDir, self).__init__(message)

def init_log(config):
    loglevel = config.get("loglevel", logging.INFO)
    logformat = config.get("logformat", '%(asctime)s:%(levelname)s:%(name)s:%(message)s')
    if config.get("logfile"):
        logging.basicConfig(filename=config.get("logfile"), format=logformat, level=loglevel)
    else:
        logging.basicConfig(format=logformat, level=loglevel)

def strip_extension(lst):
    return (os.path.splitext(l)[0] for l in lst)

def init_plugins(plugindir, plugins_to_load=None):
    if plugindir and not os.path.isdir(plugindir):
        raise InvalidPluginDir(plugindir)

    if not plugindir:
        plugindir = DIR("plugins")

    logger.debug("plugindir: {0}".format(plugindir))

    if os.path.isdir(plugindir):
        pluginfiles = glob(os.path.join(plugindir, "[!_]*.py"))
        plugins = strip_extension(os.path.basename(p) for p in pluginfiles)
    else:
        # we might be in an egg; try to get the files that way
        logger.debug("trying pkg_resources")
        import pkg_resources
        try:
            plugins = strip_extension(
                pkg_resources.resource_listdir(__name__, "plugins"))
        except OSError:
            raise InvalidPluginDir(plugindir)

    hooks = {}

    oldpath = copy.deepcopy(sys.path)
    sys.path.insert(0, plugindir)

    for plugin in plugins:
        if plugins_to_load and plugin not in plugins_to_load:
            logger.debug("skipping plugin {0}, not in plugins_to_load {1}".format(plugin, plugins_to_load))
            continue

        logger.debug("plugin: {0}".format(plugin))
        try:
            mod = importlib.import_module(plugin)
            modname = mod.__name__
            for hook in re.findall(r"\bon_(\w+)", " ".join(dir(mod))):
                hookfun = getattr(mod, "on_" + hook)
                logger.debug("plugin: attaching %s hook for %s", hook, modname)
                hooks.setdefault(hook, []).append(hookfun)

            if mod.__doc__:
                firstline = mod.__doc__.split('\n')[0]
                hooks.setdefault('help', {})[modname] = firstline
                hooks.setdefault('extendedhelp', {})[modname] = mod.__doc__

        # bare except, because the modules could raise any number of errors
        # on import, and we want them not to kill our server
        except:
            logger.warning("import failed on module {0}, module not loaded".format(plugin))
            logger.warning("{0}".format(sys.exc_info()[0]))
            logger.warning("{0}".format(traceback.format_exc()))

    sys.path = oldpath
    return hooks

def run_hook(hooks, hook, *args):
    responses = []
    for hook in hooks.get(hook, []):
        try:
            h = hook(*args)
            if h:
                responses.append(h)
        except:
            logger.warning("Failed to run plugin {0}, module not loaded".format(hook))
            logger.warning("{0}".format(sys.exc_info()[0]))
            logger.warning("{0}".format(traceback.format_exc()))

    return responses

def get_user_id_from_message(msg, msgtype):
    try:
        if msgtype == "bot_message":
            return msg["bot_id"]
        if msgtype == "message_changed":
            return msg["message"]["user"]
        if msgtype == "message_deleted":
            return msg["previous_message"]["user"]
        return msg["user"]
    except KeyError:
            return None

def handle_message(event, server):
    # plain mesages don't have a subtype; message_changed, bot_message,
    # message_deleted et al do. use the subtype if available, otherwise
    # just message
    subtype = event.get("subtype", "message")
    user_id = get_user_id_from_message(event, subtype)

    # skip messages from ourselves and from slackbot to prevent message loops
    if not user_id or user_id == server.slack.userid or user_id == "USLACKBOT":
        logger.info("skipping message {} no user found or user is "
            "self".format(event))
        return
    return "\n".join(run_hook(server.hooks, subtype, event, server))

def basic_handle(event_name):
    def handlefunc(event, server):
        return "\n".join(run_hook(server.hooks, event_name, event, server))
    return handlefunc

event_handlers = {
    "message": handle_message,
    "member_joined_channel": basic_handle("member_joined_channel"),
    "member_left_channel": basic_handle("member_left_channel"),
}

def handle_event(event, server):
    handler = event_handlers.get(event.get("type"))
    if handler:
        return handler(event, server)

def getif(config, name, envvar):
    if envvar in os.environ:
        config[name] = os.environ.get(envvar)

def init_config():
    config = {}
    getif(config, "token", "SLACK_TOKEN")
    getif(config, "loglevel", "LIMBO_LOGLEVEL")
    getif(config, "logfile", "LIMBO_LOGFILE")
    getif(config, "logformat", "LIMBO_LOGFORMAT")
    getif(config, "plugins", "LIMBO_PLUGINS")

    return config

def loop(server, test_loop=None):
    """Run the main loop

    server is a limbo Server object
    test_loop, if present, is a number of times to run the loop
    """
    try:
        loops_without_activity = 0
        while test_loop is None or test_loop > 0:
            start = time.time()
            loops_without_activity += 1

            events = server.slack.rtm_read()
            for event in events:
                loops_without_activity = 0
                logger.debug("got {0}".format(event))
                response = handle_event(event, server)
                while response:
                    # The Slack API documentation says:
                    #
                    # Clients should limit messages sent to channels to 4000
                    # characters, which will always be under 16k bytes even
                    # with a message comprised solely of non-BMP Unicode
                    # characters at 4 bytes each.
                    #
                    # but empirical testing shows that I'm getting disconnected
                    # at 4000 characters and even quite a bit lower. Use 1000
                    # to be safe
                    server.slack.rtm_send_message(event["channel"], response[:1000])
                    response = response[1000:]

            # Run the loop hook. This doesn't send messages it receives,
            # because it doesn't know where to send them. Use
            # server.slack.post_message to send messages from a loop hook
            run_hook(server.hooks, "loop", server)

            # The Slack RTM API docs say:
            #
            # > When there is no other activity clients should send a ping
            # > every few seconds
            #
            # So, if we've gone >5 seconds without any activity, send a ping.
            # If the connection has broken, this will reveal it so slack can
            # quit
            if loops_without_activity > 5:
                server.slack.ping()
                loops_without_activity = 0

            end = time.time()
            runtime = start - end
            time.sleep(max(1 - runtime, 0))

            if test_loop:
                test_loop -= 1
    except KeyboardInterrupt:
        if os.environ.get("LIMBO_DEBUG"):
            import ipdb
            ipdb.set_trace()
        raise

def relevant_environ():
    return dict((key, os.environ[key])
                for key in os.environ
                if key.startswith("SLACK") or key.startswith("LIMBO"))

def init_server(args, config, Server=LimboServer, Client=SlackClient):
    init_log(config)
    logger.debug("config: {0}".format(config))
    db = init_db(args.database_name)

    config_plugins = config.get("plugins")
    plugins_to_load = config_plugins.split(",") if config_plugins else []

    hooks = init_plugins(args.pluginpath, plugins_to_load)
    try:
        slack = Client(config["token"])
    except KeyError:
        logger.error("""Unable to find a slack token. The environment variables
limbo sees are:
{0}

and the current config is:
{1}

Try setting your bot's slack token with:

export SLACK_TOKEN=<your-slack-bot-token>
""".format(relevant_environ(), config))
        raise
    server = Server(slack, config, hooks, db)
    return server

# decode a string. if str is a python 3 string, do nothing.
def decode(str_, codec='utf8'):
    if PYTHON3:
        return str_
    else:
        return str_.decode(codec)

# encode a string. if str is a python 3 string, do nothing.
def encode(str_, codec='utf8'):
    if PYTHON3:
        return str_
    else:
        return str_.encode(codec)

def main(args):
    config = init_config()
    if args.test:
        init_log(config)
        return repl(FakeServer(), args)
    elif args.command is not None:
        init_log(config)
        cmd = decode(args.command)
        print(run_cmd(cmd, FakeServer(), args.hook, args.pluginpath, config.get("plugins")))
        return

    server = init_server(args, config)

    try:
        server.slack.rtm_connect()
        # run init hook. This hook doesn't send messages to the server (ought it?)
        run_hook(server.hooks, "init", server)

        loop(server)
    except SlackConnectionError:
        logger.warn("Unable to connect to Slack. Bad network?")
        raise
    except SlackLoginError:
        logger.warn("Login Failed, invalid token <{0}>?".format(config["token"]))
        raise

# run a command. cmd should be a unicode string (str in python3, unicode in python2).
# returns a string appropriate for printing (str in py2 and py3)
def run_cmd(cmd, server, hook, pluginpath, plugins_to_load):
    server.hooks = init_plugins(pluginpath, plugins_to_load)
    event = {'type': hook, 'text': cmd, "user": "2", 'ts': time.time(), 'team': None, 'channel': 'repl_channel'}
    return encode(handle_event(event, server))

# raw_input in 2.6 is input in python 3. Set `input` to the correct function
try:
    input = raw_input
except NameError:
    pass

def repl(server, args):
    try:
        while 1:
            cmd = decode(input("limbo> "))
            if cmd.lower() == "quit" or cmd.lower() == "exit":
                return

            print(run_cmd(cmd, server, args.hook, args.pluginpath, None))
    except (EOFError, KeyboardInterrupt):
        print()
        pass

def init_db(database_file):
    return sqlite3.connect(database_file)



"""!stock <search term> return a stock photo for <search term>"""

from random import shuffle
import re
try:
    from urllib import quote
except ImportError:
    from urllib.request import quote

import requests
from bs4 import BeautifulSoup

def stock(searchterm):
    searchterm = quote(searchterm)
    url = "http://www.shutterstock.com/cat.mhtml?searchterm={0}&search_group=&lang=en&language=en&search_source=search_form&version=llv1".format(searchterm)
    r = requests.get(url)
    soup = BeautifulSoup(r.text, "html5lib")
    images = [x["src"] for x in soup.select(".gc_clip img")]
    shuffle(images)

    return images[0] if images else ""

def on_message(msg, server):
    text = msg.get("text", "")
    match = re.findall(r"!stock (.*)", text)
    if not match:
        return

    return stock(match[0].encode("utf8"))

on_bot_message = on_message
